@@ -15,19 +15,20 @@ Loading it makes AnyEvent check all arguments to AnyEvent-methods, at the
 expense of being slower (often the argument checking takes longer than the
 actual function).
 
+B<< Currently, only AnyEvent I<methods> are checked, the AE:: I<functions> are not
+affected. >>
+
 Normally, you don't load this module yourself but instead use it
 indirectly via the C<PERL_ANYEVENT_STRICT> environment variable (see
-L<AnyEvent>). However, this module can be loaded at any time.
+L<AnyEvent>). However, this module can be loaded manually at any time.
 
 =cut
 
 package AnyEvent::Strict;
 
 use Carp qw(croak);
-use Fcntl ();
 
 use AnyEvent (); BEGIN { AnyEvent::common_sense }
-use AnyEvent::Util ();
 
 our @ISA;
 
@@ -36,7 +37,7 @@ AnyEvent::post_detect {
    # # and link us in before it in the chain.
    my $MODEL = shift @AnyEvent::ISA;
    unshift @ISA, $MODEL;
-   unshift @AnyEvent::ISA, AnyEvent::Strict::
+   unshift @AnyEvent::ISA, AnyEvent::Strict::;
 };
 
 sub io {
@@ -50,11 +51,11 @@ sub io {
    $arg{poll} =~ /^[rw]$/
       or croak "AnyEvent->io called with illegal poll argument '$arg{poll}'";
 
-   if (defined fileno $arg{fh} or ref $arg{fh} or $arg{fh} !~ /^\s*\d+\s*$/) {
-      defined fileno $arg{fh}
-         or croak "AnyEvent->io called with illegal fh argument '$arg{fh}'";
-   } else {
+   if ($arg{fh} =~ /^\s*\d+\s*$/) {
       $arg{fh} = AnyEvent::_dupfh $arg{poll}, $arg{fh};
+   } else {
+      defined eval { fileno $arg{fh} }
+         or croak "AnyEvent->io called with illegal fh argument '$arg{fh}'";
    }
 
    -f $arg{fh}
@@ -99,7 +100,7 @@ sub signal {
       or croak "AnyEvent->signal called with illegal cb argument '$arg{cb}'";
    delete $arg{cb};
  
-   defined AnyEvent::Util::sig2num $arg{signal}
+   defined AnyEvent::Base::sig2num $arg{signal} and $arg{signal} == 0
       or croak "AnyEvent->signal called with illegal signal name '$arg{signal}'";
    delete $arg{signal};
  
